----------------------------------------------------------------------------------
-- Simulacija krmilnika VGA - test bench
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity top_tb is
    --  Port ( );
end entity;

architecture Behavioral of top_tb is
    ----------------------------------------------------------------------------------
    -- KONSTANTE
    ----------------------------------------------------------------------------------
    constant clock_period : time := 10ns;

    ----------------------------------------------------------------------------------
    -- KOMPONENTE
    ----------------------------------------------------------------------------------
    -- unit under test
    component top is
        port (
            CLK100MHZ:  in  std_logic;
            CPU_RESETN: in  std_logic;
            VGA_HS:     out std_logic;
            VGA_VS:     out std_logic;
            VGA_R:      out std_logic_vector (3 downto 0);
            VGA_G:      out std_logic_vector (3 downto 0);
            VGA_B:      out std_logic_vector (3 downto 0));
    end component;


    ----------------------------------------------------------------------------------
    -- NOTRANJI SIGNALI
    ----------------------------------------------------------------------------------
    -- Signali, ki povezujejo vhode UUT z generatorjem stimulusov
    signal clk_sim, reset_sim : std_logic := '0';
    -- Signali, ki povezujejo izhode UUT z osciloskopom (waveform)
    signal hsync_sim, vsync_sim : std_logic := '0';
    signal red_sim, green_sim, blue_sim: std_logic_vector(3 downto 0) := "0000";

begin
    -- instanciacija modula, ki ga testiramo
    UUT: top
        port map(
            CLK100MHZ  => clk_sim,
            CPU_RESETN => reset_sim,
            VGA_HS     => hsync_sim,
            VGA_VS     => vsync_sim,
            VGA_R      => red_sim,
            VGA_G      => green_sim,
            VGA_B      => blue_sim
        );

    -- Generator stimulusov (draljajev) = scenarij poteka signalov 
    -- 1. Ura 
    clk_stimuli: process
    begin
        clk_sim <= '0';
        wait for clock_period/2;
        clk_sim <= '1';
        wait for clock_period/2;
        -- alternativa:
        --clk_sim <= not clk_sim;
        --wait for clock_period/2;
    end process;

    -- 2. Ostali signali
    other_stimuli: process
    begin
        -- dolgi reset (za 3 u.p.)
        -- Reset je aktiven, ko je 0 (CPU_RESETN)
        reset_sim <= '0';
        wait for clock_period*3;

        -- reset umaknemo
        reset_sim <= '1';
        -- akaj v neskonnost
        wait;
    end process;
end Behavioral;
