----------------------------------------------------------------------------------
-- Krmilnik za VGA: glavni modul
-- Create Date: 2.11.2022
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity top is
    port (
        CLK100MHZ:  in  std_logic;
        CPU_RESETN: in  std_logic; -- pozor: negativna logika (ko je pritisnjen, je '0')
        VGA_HS:     out std_logic;
        VGA_VS:     out std_logic;
        VGA_R:      out std_logic_vector (3 downto 0);
        VGA_G:      out std_logic_vector (3 downto 0);
        VGA_B:      out std_logic_vector (3 downto 0));
end top;

architecture Behavioral of top is
    ----------------------------------------------------------------------------------
    -- KONSTANTE
    ----------------------------------------------------------------------------------

    ----------------------------------------------------------------------------------
    -- KOMPONENTE
    ----------------------------------------------------------------------------------
    component hsyncModule is
        port (
            clk:      in  std_logic;
            reset:    in  std_logic;
            ce:       out std_logic;
            display:  out std_logic;
            column:   out unsigned(9 downto 0);
            hsync:    out std_logic);
    end component;

    component vsyncModule is
        port (
            clk:     in std_logic;
            reset:   in std_logic;
            ce:      in std_logic;
            display: out std_logic;
            row:     out unsigned(9 downto 0);
            vsync:   out std_logic);
    end component;

    ----------------------------------------------------------------------------------
    -- NOTRANJI SIGNALI
    ----------------------------------------------------------------------------------
    signal count_enable: std_logic;
    signal display_h, display_v, display: std_logic;
    signal column, row: unsigned(9 downto 0);
    signal border: std_logic; -- Kdaj smo na robu zaslona
    
begin
    ----------------------------------------------------------------------------------
    -- INSTANCE MODULOV, POVEZOVANJE
    ----------------------------------------------------------------------------------
    hsync_inst: hsyncModule
        port map(
            clk     =>  CLK100MHZ,
            reset   =>  not CPU_RESETN,
            ce      =>  count_enable,
            display =>  display_h,
            column  =>  column,
            hsync   =>  VGA_HS
        );

    vsync_inst: vsyncModule
        port map(
            clk     =>  CLK100MHZ,
            reset   =>  not CPU_RESETN,
            ce      =>  count_enable,
            display =>  display_v,
            row     =>  row,
            vsync   =>  VGA_VS
        );
    ----------------------------------------------------------------------------------
    -- Logika za prikazovanje na zaslonu 
    ----------------------------------------------------------------------------------
    -- (TODO: vrednosti slikovnih tok preberi iz video RAM)

    -- Kdaj lahko aktiviramo el. topove
    display <= display_h and display_v;

    -- Primer #1: celoten zaslon bel 
    -- El. topovi morajo biti ugasnjeni, ko nismo v vidnem delu slike
    --VGA_R <= "1111" when display='1' else "0000";
    --VGA_G <= "1111" when display='1' else "0000";
    --VGA_B <= "1111" when display='1' else "0000";

    -- Primer #2: bel rob 
    border <= '1' when display='1' and (row=0 or row=479 or column=0 or column=639) else '0';
    VGA_R <= "1111" when border='1' else "0000";
    VGA_G <= "1111" when border='1' else "0000";
    VGA_B <= "1111" when border='1' else "0000";



end Behavioral;
